using System;
using System.Collections.Generic;
using System.Text;
using System.Drawing;
using System.IO;

namespace csWallChange
{
    /// <summary>
    /// Has static methods to help in creation of wallpaper.
    /// </summary> 
    class WallpaperHelper
    {

        internal const int calendarWidth = 160;
        internal const int calendarHeight = 120;
        internal const int deltasPerSlice = 4;
        internal const double percentFromCalendarOK = 0.6;


        private static Color sundayColor, normalColor;

        /// <summary>
        /// Finds the optimum place to draw the calendar on this wallpaper
        /// </summary>
        /// <param name="bmp">The wallpaper on which to draw the calendar</param>
        /// <returns>A point on which to draw the calendar</returns>
        internal static Point findCalendarLocation(Bitmap bmp) {
            //Get the dimensions
            int width = bmp.Width;
            int height = bmp.Height - 30; //30 pixels for the taskbar

            #region Coding Comments
            //Decide on the number of subdivisions and the percentage of the width from 
            //which to allow calendar to be placed.
            
            //Experience shows that a typical wallpaper needs 160 x 120 pixels
            //So we take divisions of 40 pixels each on the x-axis, and 30 pixels on the y-axis

            //We allow the wallpaper to appear from 60% of the width


            //                  |<--------- 60% -------->|
            //
            //  +----------------------------------------+
            //  |               |xxxxxxxxxxxxxxxxxxxxxxxx|
            //  |               |xxxxxxxxxxxxxxxxxxxxxxxx|
            //  |               |xxxxxxxxxxxxxxxxxxxxxxxx|
            //  |               |xxx                  xxx|
            //  |               |xxx  Allowed region  xxx|
            //  |               |xxx                  xxx|
            //  |               |xxxxxxxxxxxxxxxxxxxxxxxx|
            //  |               |xxxxxxxxxxxxxxxxxxxxxxxx|
            //  |               |xxxxxxxxxxxxxxxxxxxxxxxx|
            //  |               |xxxxxxxxxxxxxxxxxxxxxxxx|
            //  +----------------------------------------+
            //
            //  |<------------ Screen Width ------------>|

            #endregion

            int xdelta = calendarWidth/deltasPerSlice;
            int ydelta = calendarWidth/deltasPerSlice;
            int xnum = (int)((percentFromCalendarOK * width) / xdelta);//number of slices in the x-axis
            int ynum = (int)(height / ydelta);


            
            double[,] sd = new double[xnum, ynum];
            for (int i = 0; i < xnum; i++) {
                for (int j = 0; j < ynum; j++) {
                    sd[i,j] = getStandardDeviation((width - (xnum - i)*xdelta),
                        (height - (ynum-j)*ydelta), xdelta, ydelta, bmp);
                }
            }

            //NOTICE: micro-optimisation possible in the next block of code.
            //Can cache a block of 3 deltas in each axis

            //One calendar will require 4 deltas to be displayed.
            int xpos = 0, ypos = 0;
            double sdt = double.MaxValue;
            double t = 0;
            for (int i1 = 0; i1 < xnum - deltasPerSlice; i1++) {
                for (int i2 = 0; i2 < ynum - deltasPerSlice; i2++) {

                    t = 0;
                    //check a slice of 4 deltas
                    for (int i3 = 0; i3 < deltasPerSlice; i3++) {
                        for (int i4 = 0; i4 < deltasPerSlice; i4++) {
                            t += sd[i1 + i3, i2 + i4];
                        }
                    }

                    //if this is less than the current minima
                    if (t < sdt) {
                        sdt = t;
                        xpos = (width - (xnum - i1) * xdelta);
                        ypos = (height - (ynum - i2) * ydelta);
                    }

                }
            }


            return new Point(xpos, ypos);

        }

        private static double getStandardDeviation(int xstart, int ystart, int xdelta, int ydelta, System.Drawing.Bitmap bmp) {
            int n = 0;

            double rsum, gsum, bsum;
            double rsquare, gsquare, bsquare;
            Color ct;

            rsum = gsum = bsum = 0;
            rsquare = gsquare = bsquare = 0;

            //iterate over the pic to find the sd.
            for (int i = 0; i < xdelta; i++) {
                for (int j = 0; j < ydelta; j++) {
                    ct = bmp.GetPixel(xstart + i, ystart + j);

                    rsum += ct.R;
                    rsquare += ct.R * ct.R;

                    gsum += ct.G;
                    gsquare += ct.G * ct.G;

                    bsum += ct.B;
                    bsquare += ct.B * ct.B;

                    n++;
                }
            }

            double redSD = (rsquare / n) - (rsum / n) * (rsum / n);
            double greenSD = (gsquare / n) - (gsum / n) * (gsum / n);
            double blueSD = (bsquare / n) - (bsum / n) * (bsum / n);

            return (redSD + greenSD + blueSD) / 3.0;

        }



        internal static void drawCalendar(Bitmap bmp, Point p) {
            Graphics g = Graphics.FromImage(bmp);

            //Draw a white rectangle to keep track of things - debugging
            //g.DrawRectangle(Pens.White, new Rectangle(p, new Size(160, 120)));

            //Get the general color values.
            double redContent = 0, greenContent = 0, blueContent = 0;
            getColorAverages(bmp, ref redContent, ref greenContent, ref blueContent, p);

            Brush normalBrush = null;
            Brush sundayBrush;

            if (redContent + greenContent + blueContent > 1.5) {
                normalBrush = Brushes.Black;
                normalColor = Color.Black;
            }
            else {
                normalBrush = Brushes.White;
                normalColor = Color.White;
            }

            if (redContent > blueContent + 0.2) {
                sundayBrush = Brushes.Blue;
                sundayColor = Color.Blue;
            }
            else {
                sundayBrush = Brushes.Red;
                sundayColor = Color.Red;
            }


            writeCalendarText(p, sundayBrush, normalBrush, g);
            


        }

        private static void writeCalendarText(Point p, Brush sundayBrush, Brush normalBrush, Graphics g) {
            //g.FillEllipse(sundayBrush, new Rectangle(p, new Size(30,40)));

            //Compute the different params
            int topx = p.X;
            int topy = p.Y;
            int gapx = 24;
            int gapy = 20;
            Font font = new Font("Verdana", 8);

            //Write down the weekday headers.
            string[] weekDayText = new string[]{
                "Su", "M", "T", "W", "Th", "F", "S"};
            g.DrawString(weekDayText[0], font, sundayBrush, p.X + 0 * gapx, p.Y);
            for (int i = 1; i < 7; i++) {
                g.DrawString(weekDayText[i], font, normalBrush, p.X + i * gapx, p.Y);
            }

            //Get all the 'date' params.
            int totalDaysInMonth = 31;
            int month = 0;//reserved for future use
            int year = 0;//reserved for future use
            int dayToday = 0;
            int dayOfWeekToday = 0;

            DateTime d = DateTime.Today;
            dayToday = d.Day;
            month = d.Month;
            year = d.Year;

            //Now go to start of month
            d = new DateTime(year, month, 1);

            dayOfWeekToday = (int)d.DayOfWeek;
            if (month == 4 || month == 6 || month == 9 || month == 11) totalDaysInMonth = 30;
            if (month == 2) {
                totalDaysInMonth = 28;
                if ((year % 4 == 0) && (year % 400 != 0)) totalDaysInMonth = 29;
            }

            int cursorX = 0, cursorY = 1; //where the current date printing cursor is
            cursorX = dayOfWeekToday;
            for (int i = 0; i < totalDaysInMonth; i++) {

                //draw today's date, and the highlight around today's date
                if (i + 1 == dayToday) {
                    switch (csWallChange.Properties.Settings.Default.TodayHighlightMethod) {
                        case "Underline": {
                                Font f2 = new Font(font, FontStyle.Underline);
                                if (cursorX == 0)
                                    g.DrawString((i + 1) + "", f2, sundayBrush, cursorX * gapx + p.X,
                                                        cursorY * gapy + p.Y);
                                else
                                    g.DrawString((i + 1) + "", f2, normalBrush, cursorX * gapx + p.X,
                                                        cursorY * gapy + p.Y);
                            }
                            break;

                        case "Bold": {
                                Font f2 = new Font(font, FontStyle.Bold);
                                if (cursorX == 0)
                                    g.DrawString((i + 1) + "", f2, sundayBrush, cursorX * gapx + p.X,
                                                        cursorY * gapy + p.Y);
                                else
                                    g.DrawString((i + 1) + "", f2, normalBrush, cursorX * gapx + p.X,
                                                        cursorY * gapy + p.Y);
                            }
                            break;
                        case "Italics": {
                                Font f2 = new Font(font, FontStyle.Italic);
                                if (cursorX == 0)
                                    g.DrawString((i + 1) + "", f2, sundayBrush, cursorX * gapx + p.X,
                                                        cursorY * gapy + p.Y);
                                else
                                    g.DrawString((i + 1) + "", f2, normalBrush, cursorX * gapx + p.X,
                                                        cursorY * gapy + p.Y);
                            }
                            break;
                        default:
                            if (cursorX == 0)
                                g.DrawString((i + 1) + "", font, sundayBrush, cursorX * gapx + p.X,
                                                    cursorY * gapy + p.Y);
                            else
                                g.DrawString((i + 1) + "", font, normalBrush, cursorX * gapx + p.X,
                                                    cursorY * gapy + p.Y);
                            break;
                    }
                    drawHighlight(g, cursorX * gapx + p.X, cursorY * gapy + p.Y, cursorX == 0);
                }
                //Draw the text for that date
                else if (cursorX == 0) {
                    g.DrawString((i + 1) + "", font, sundayBrush, cursorX * gapx + p.X,
                        cursorY * gapy + p.Y);
                }
                else {
                    g.DrawString((i + 1) + "", font, normalBrush, cursorX * gapx + p.X,
                        cursorY * gapy + p.Y);
                }



                cursorX++;
                if (cursorX > 6) {
                    cursorX = 0;
                    cursorY++;
                }
            }




        }

        private static void drawHighlight(Graphics g, int x, int y,
                bool isSunday) {
            Pen p = null;
            if (isSunday)
                p = new Pen(sundayColor);
            else
                p = new Pen(normalColor);


            switch (csWallChange.Properties.Settings.Default.TodayHighlightMethod) {
                case "Box":
                    g.DrawRectangle(p, x, y, 18, 13);
                    break;
                case "Circle":
                    g.DrawEllipse(p, x-1, y-1, 20, 15);
                    break;
                default:
                    break;
            }
        }



        private static void getColorAverages(Bitmap bmp, ref double redContent, ref double greenContent, ref double blueContent, Point p) {
            int n = 0;
            Color c;
            redContent = greenContent = blueContent = 0;

            for (int i = p.X; i < p.X+calendarWidth; i++) {
                for (int j = p.Y; j < p.Y+calendarHeight; j++) {
                    c = bmp.GetPixel(i, j);
                    redContent += (c.R / 256.0);
                    greenContent += (c.G / 256.0);
                    blueContent += (c.B / 256.0);
                    n++;
                }
            }

            redContent /= n;
            greenContent /= n;
            blueContent /= n;
        }


        public static string getRandomWallpaper() {
            //Where to start searching...
            string srchFolder = csWallChange.Properties.Settings.Default.SearchFolder;

            string[] exts = new string[] { "*.jpg", "*.bmp", "*.png", "*.gif" };
            int[] numExts = new int[exts.Length];
            int numFiles = 0;
            string[][] sarr = new string[exts.Length][];

            for (int i = 0; i < exts.Length; i++) {
                if(csWallChange.Properties.Settings.Default.SearchSubFolders)
                    sarr[i] = Directory.GetFiles(srchFolder, exts[i], SearchOption.AllDirectories);
                else
                    sarr[i] = Directory.GetFiles(srchFolder, exts[i], SearchOption.TopDirectoryOnly);
                numExts[i] = sarr[i].Length;
                numFiles += numExts[i];
            }


            if (numFiles < 1) return null;

            Random r = new Random();
            int n = r.Next(numFiles);
            int wheel = 0;

            for (int i = 0; i < exts.Length; i++) {
                wheel += numExts[i];
                if(n<wheel)
                    return sarr[i][wheel-n-1];
            }

            //it will never reach here, if it does, return the last file.
            return sarr[exts.Length - 1][sarr[exts.Length - 1].Length - 1];
            
        }


        private const double epsa = 0.05;

        internal static void placeWallpaper(Bitmap org, Bitmap bmp) {
            
            //find the aspect ratio of both..
            double aspectOld = org.Width / (double)org.Height;
            double aspectNew = bmp.Width / (double)bmp.Height;

            Graphics g = Graphics.FromImage(bmp);

            if (Math.Abs(aspectNew - aspectOld) < epsa) {
                //simply draw it..
                g.DrawImage(org, 0, 0, bmp.Width, bmp.Height);
            }

            else {
                //we fill it with the top left pixel
                g.FillRectangle(new SolidBrush(org.GetPixel(0, 0)), 0, 0, bmp.Width, bmp.Height);

                //we draw the image in the middle, scaled as much as possible.
                double scaleFactor = bmp.Width / (double)org.Width;
                scaleFactor = (bmp.Height / (double)org.Height)<scaleFactor?(bmp.Height / (double)org.Height):scaleFactor;

                int destWidth = (int)(org.Width * scaleFactor);
                int destHeight = (int)(org.Height * scaleFactor);

                int destX = bmp.Width / 2 - destWidth / 2;
                int destY = bmp.Height / 2 - destHeight / 2;

                g.DrawImage(org, destX, destY, destWidth, destHeight);

            }
                


        }



    }
}
